# Generated by Django 2.1.7 on 2019-04-23 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("nuntius", "0012_campaignsentevent_tracking_id")]

    operations = [
        migrations.AlterField(
            model_name="campaignsentevent",
            name="esp_message_id",
            field=models.CharField(
                editable=False,
                max_length=255,
                null=True,
                unique=True,
                verbose_name="ID given by the sending server",
            ),
        )
    ]
